@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp 15585 2008-10-23 14:57:27Z warmerdam $
+ * $Id: grass57dataset.cpp 15584 2008-10-23 14:53:15Z warmerdam $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -56,7 +56,7 @@ char *GPJ_grass_to_wkt(struct Key_Value *proj_info,
 
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp 15585 2008-10-23 14:57:27Z warmerdam $");
+CPL_CVSID("$Id: grass57dataset.cpp 15584 2008-10-23 14:53:15Z warmerdam $");
 
 CPL_C_START
 void	GDALRegister_GRASS(void);
@@ -921,7 +921,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     
     if( G_get_cellhd( papszCells[0], papszMapsets[0], &(poDS->sCellInfo) ) != 0 ) {
 	CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster header");
-	/* TODO: delete poDS ? */
+	delete poDS;
 	return NULL;
     }
 
@@ -944,6 +944,8 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 	projinfo = G_get_projinfo();
 	projunits = G_get_projunits();
         poDS->pszProjection = GPJ_grass_to_wkt ( projinfo, projunits, 0, 0);
+        G_free_key_value(projinfo);
+        G_free_key_value(projunits);
     }
 
 /* -------------------------------------------------------------------- */
@@ -956,13 +958,17 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
 	if ( !rb->valid ) {
 	    CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster band %d", iBand);
-	    // TODO: delete poDS ?
+	    delete rb;
+	    delete poDS;
 	    return NULL;
 	}
 
         poDS->SetBand( iBand+1, rb );
     }
 
+    CSLDestroy(papszCells);
+    CSLDestroy(papszMapsets);
+
     return poDS;
 }
 
