@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgrassdatasource.cpp 11338 2007-04-23 09:45:44Z mloskot $
+ * $Id: ogrgrassdatasource.cpp 15235 2008-08-27 19:44:56Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRGRASSDataSource class.
@@ -31,7 +31,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrgrassdatasource.cpp 11338 2007-04-23 09:45:44Z mloskot $");
+CPL_CVSID("$Id: ogrgrassdatasource.cpp 15235 2008-08-27 19:44:56Z rouault $");
 
 /************************************************************************/
 /*                         Grass2CPLErrorHook()                         */
@@ -58,6 +58,7 @@ OGRGRASSDataSource::OGRGRASSDataSource()
     pszMap = NULL;
     papoLayers = NULL;
     nLayers = 0;
+    bOpened = FALSE;
 }
 
 /************************************************************************/
@@ -74,6 +75,9 @@ OGRGRASSDataSource::~OGRGRASSDataSource()
     if ( pszLocation ) CPLFree( pszLocation );
     if ( pszMapset ) CPLFree( pszMapset );
     if ( pszMap ) CPLFree( pszMap );
+    
+    if (bOpened)
+        Vect_close(&map);
 }
 
 /************************************************************************/
@@ -211,6 +215,8 @@ int OGRGRASSDataSource::Open( const char * pszNewName, int bUpdate,
 	papoLayers[nLayers++] = poLayer;
     }
     
+    bOpened = TRUE;
+    
     return TRUE;
 }
 
